@@ -123,17 +123,18 @@ def fix_inversion(mesh, multibody=False):
                 flip[faces] = True
         # one or more faces needs flipping
         if flip.any():
-            with mesh._cache:
-                # flip normals of necessary faces
-                if 'face_normals' in mesh._cache:
-                    mesh.face_normals[flip] *= -1.0
-                # flip faces
-                mesh.faces[flip] = np.fliplr(mesh.faces[flip])
-            # save wangled normals
-            mesh._cache.clear(exclude=['face_normals'])
+            # flip normals of necessary faces
+            if 'face_normals' in mesh._cache:
+                normals = mesh.face_normals.copy()
+                normals[flip] *= -1.0
+            else:
+                normals = None
+            # flip faces
+            mesh.faces[flip] = np.fliplr(mesh.faces[flip])
+            if normals is not None:
+                mesh.face_normals = normals
 
     elif mesh.volume < 0.0:
-        # reverse every triangles and flip every normals
         mesh.invert()
 
 
@@ -256,10 +257,9 @@ def fill_holes(mesh):
     g = nx.from_edgelist(
         np.column_stack((boundary_edges,
                          index_as_dict)))
-    cycles = np.array(nx.cycle_basis(g))
     new_faces = []
     new_vertex = []
-    for hole in cycles:
+    for hole in nx.cycle_basis(g):
         # convert the hole, which is a polygon of vertex indices
         # to triangles and new vertices
         faces, vertex = hole_to_faces(hole=hole)
