@@ -8,6 +8,7 @@ import numpy as np
 
 from . import util
 
+from .util import unitize, diagonal_dot
 from .points import point_plane_distance
 from .constants import tol
 
@@ -39,14 +40,15 @@ def area(triangles=None, crosses=None, sum=False):
     ----------
     triangles : (n, 3, 3) float
       Vertices of triangles
+    crosses : (n, 3) float or None
+      As a speedup don't re- compute cross products
     sum : bool
       Return summed area or individual triangle area
 
     Returns
     ----------
-    area:
-        if sum: float, sum area of triangles
-        else:   (n,) float, individual area of triangles
+    area : (n,) float or float
+      Individual or summed area depending on `sum` argument
     """
     if crosses is None:
         crosses = cross(triangles)
@@ -77,7 +79,7 @@ def normals(triangles=None, crosses=None):
     if crosses is None:
         crosses = cross(triangles)
     # unitize the cross product vectors
-    unit, valid = util.unitize(crosses, check_valid=True)
+    unit, valid = unitize(crosses, check_valid=True)
     return unit, valid
 
 
@@ -93,25 +95,30 @@ def angles(triangles):
     Returns
     ------------
     angles : (n, 3) float
-      Angles at vertex positions, in radians
+      Angles at vertex positions in radians
+      Degenerate angles will be returned as zero
     """
+    # don't copy triangles
+    triangles = np.asanyarray(triangles, dtype=np.float64)
 
-    # get a vector for each edge of the triangle
-    u = triangles[:, 1] - triangles[:, 0]
-    v = triangles[:, 2] - triangles[:, 0]
-    w = triangles[:, 2] - triangles[:, 1]
+    # get a unit vector for each edge of the triangle
+    u = unitize(triangles[:, 1] - triangles[:, 0])
+    v = unitize(triangles[:, 2] - triangles[:, 0])
+    w = unitize(triangles[:, 2] - triangles[:, 1])
 
-    # normalize each vector in place
-    u /= np.linalg.norm(u, axis=1, keepdims=True)
-    v /= np.linalg.norm(v, axis=1, keepdims=True)
-    w /= np.linalg.norm(w, axis=1, keepdims=True)
+    # run the cosine and per-row dot product
+    result = np.zeros((len(triangles), 3), dtype=np.float64)
+    # clip to make sure we don't float error past 1.0
+    result[:, 0] = np.arccos(np.clip(diagonal_dot(u, v), -1, 1))
+    result[:, 1] = np.arccos(np.clip(diagonal_dot(-u, w), -1, 1))
+    # the third angle is just the remaining
+    result[:, 2] = np.pi - result[:, 0] - result[:, 1]
 
-    # run the cosine and an einsum that definitely does something
-    a = np.arccos(np.clip(np.einsum('ij, ij->i', u, v), -1, 1))
-    b = np.arccos(np.clip(np.einsum('ij, ij->i', -u, w), -1, 1))
-    c = np.pi - a - b
+    # a triangle with any zero angles is degenerate
+    # so set all of the angles to zero in that case
+    result[(result < tol.merge).any(axis=1), :] = 0.0
 
-    return np.column_stack([a, b, c])
+    return result
 
 
 def all_coplanar(triangles):
@@ -130,7 +137,7 @@ def all_coplanar(triangles):
     """
     triangles = np.asanyarray(triangles, dtype=np.float64)
     if not util.is_shape(triangles, (-1, 3, 3)):
-        raise ValueError('Triangles must be (n,3,3)!')
+        raise ValueError('Triangles must be (n, 3, 3)!')
 
     test_normal = normals(triangles)[0]
     test_vertex = triangles[0][0]
@@ -149,7 +156,7 @@ def any_coplanar(triangles):
     """
     triangles = np.asanyarray(triangles, dtype=np.float64)
     if not util.is_shape(triangles, (-1, 3, 3)):
-        raise ValueError('Triangles must be (n,3,3)!')
+        raise ValueError('Triangles must be (n, 3, 3)!')
 
     test_normal = normals(triangles)[0]
     test_vertex = triangles[0][0]
@@ -192,13 +199,14 @@ def mass_properties(triangles,
     """
     triangles = np.asanyarray(triangles, dtype=np.float64)
     if not util.is_shape(triangles, (-1, 3, 3)):
-        raise ValueError('Triangles must be (n,3,3)!')
+        raise ValueError('Triangles must be (n, 3, 3)!')
 
     if crosses is None:
         crosses = cross(triangles)
 
     # these are the subexpressions of the integral
-    f1 = triangles.sum(axis=1)
+    # this is equvilant but 7x faster than triangles.sum(axis=1)
+    f1 = triangles[:, 0, :] + triangles[:, 1, :] + triangles[:, 2, :]
 
     # for the the first vertex of every triangle:
     # triangles[:,0,:] will give rows like [[x0, y0, z0], ...]
@@ -223,12 +231,14 @@ def mass_properties(triangles,
     integral[4:7] = (crosses * f3).T
     for i in range(3):
         triangle_i = np.mod(i + 1, 3)
-        integral[i + 7] = crosses[:, i] * ((triangles[:, 0, triangle_i] * g0[:, i]) +
-                                           (triangles[:, 1, triangle_i] * g1[:, i]) +
-                                           (triangles[:, 2, triangle_i] * g2[:, i]))
+        integral[i + 7] = crosses[:, i] * (
+            (triangles[:, 0, triangle_i] * g0[:, i]) +
+            (triangles[:, 1, triangle_i] * g1[:, i]) +
+            (triangles[:, 2, triangle_i] * g2[:, i]))
 
-    coefficients = 1.0 / np.array([6, 24, 24, 24, 60, 60, 60, 120, 120, 120],
-                                  dtype=np.float64)
+    coefficients = 1.0 / np.array(
+        [6, 24, 24, 24, 60, 60, 60, 120, 120, 120],
+        dtype=np.float64)
     integrated = integral.sum(axis=1) * coefficients
 
     volume = integrated[0]
@@ -289,12 +299,19 @@ def windings_aligned(triangles, normals_compare):
       Are normals aligned with triangles
     """
     triangles = np.asanyarray(triangles, dtype=np.float64)
-    if not util.is_shape(triangles, (-1, 3, 3)):
-        raise ValueError('Triangles must be (n,3,3)!')
+    if not util.is_shape(triangles, (-1, 3, 3), allow_zeros=True):
+        raise ValueError(
+            'triangles must have shape (n, 3, 3), got %s' % str(triangles.shape))
+    normals_compare = np.asanyarray(normals_compare, dtype=np.float64)
 
     calculated, valid = normals(triangles)
-    difference = util.diagonal_dot(calculated,
-                                   normals_compare[valid])
+    if normals_compare.shape == (3,):
+        # single comparison vector case
+        difference = np.dot(calculated, normals_compare)
+    else:
+        # multiple comparison case
+        difference = diagonal_dot(
+            calculated, normals_compare[valid])
 
     aligned = np.zeros(len(triangles), dtype=np.bool)
     aligned[valid] = difference > 0.0
@@ -309,15 +326,17 @@ def bounds_tree(triangles):
 
     Parameters
     ---------
-    triangles: (n, 3, 3) list of vertices
+    triangles : (n, 3, 3) float
+      Triangles in space
 
     Returns
     ---------
-    tree: Rtree object
+    tree : rtree.Rtree
+      One node per triangle
     """
     triangles = np.asanyarray(triangles, dtype=np.float64)
     if not util.is_shape(triangles, (-1, 3, 3)):
-        raise ValueError('Triangles must be (n,3,3)!')
+        raise ValueError('Triangles must be (n, 3, 3)!')
 
     # the (n,6) interleaved bounding box for every triangle
     triangle_bounds = np.column_stack((triangles.min(axis=1),
@@ -338,16 +357,19 @@ def nondegenerate(triangles, areas=None, height=None):
 
     Parameters
     ----------
-    triangles: (n, 3, 3) float, list of triangles
-    height:    float, minimum edge of a triangle to be kept
+    triangles : (n, 3, 3) float
+      Triangles in space
+    height : float
+      Minimum edge length of a triangle to keep
 
     Returns
     ----------
-    nondegenerate: (n,) bool array of triangles that have area
+    nondegenerate : (n,) bool
+      True if a triangle meets required minimum height
     """
     triangles = np.asanyarray(triangles, dtype=np.float64)
     if not util.is_shape(triangles, (-1, 3, 3)):
-        raise ValueError('Triangles must be (n,3,3)!')
+        raise ValueError('Triangles must be (n, 3, 3)!')
 
     if height is None:
         height = tol.merge
@@ -364,20 +386,21 @@ def extents(triangles, areas=None):
     """
     Return the 2D bounding box size of each triangle.
 
-
     Parameters
     ----------
-    triangles: (n, 3, 3) float, list of triangles
-    areas:     (n,) float,      list of triangles area
-
+    triangles : (n, 3, 3) float
+      Triangles in space
+    areas : (n,) float
+      Optional area of input triangles
 
     Returns
     ----------
-    box:       (n,2) float, the size of the 2D oriented bounding box.
+    box :  (n, 2) float
+      The size of each triangle's 2D oriented bounding box
     """
     triangles = np.asanyarray(triangles, dtype=np.float64)
     if not util.is_shape(triangles, (-1, 3, 3)):
-        raise ValueError('Triangles must be (n,3,3)!')
+        raise ValueError('Triangles must be (n, 3, 3)!')
 
     if areas is None:
         areas = area(triangles=triangles,
@@ -411,19 +434,22 @@ def barycentric_to_points(triangles, barycentric):
     to cartesian points.
 
     Parameters
-    ----------
-    triangles:   (n,3,3) float, list of triangles in space
-    barycentric: (n,2) float, barycentric coordinates
+    ------------
+    triangles : (n, 3, 3) float
+      Triangles in space
+    barycentric : (n, 2) float
+      Barycentric coordinates
 
     Returns
     -----------
-    points: (m,3) float, points in space
+    points : (m, 3) float
+      Points in space
     """
     barycentric = np.asanyarray(barycentric, dtype=np.float64)
     triangles = np.asanyarray(triangles, dtype=np.float64)
 
     if not util.is_shape(triangles, (-1, 3, 3)):
-        raise ValueError('Triangles must be (n,3,3)!')
+        raise ValueError('Triangles must be (n, 3, 3)!')
     if barycentric.shape == (2,):
         barycentric = np.ones((len(triangles), 2),
                               dtype=np.float64) * barycentric
@@ -472,22 +498,22 @@ def points_to_barycentric(triangles,
 
     def method_cross():
         n = np.cross(edge_vectors[:, 0], edge_vectors[:, 1])
-        denominator = util.diagonal_dot(n, n)
+        denominator = diagonal_dot(n, n)
 
         barycentric = np.zeros((len(triangles), 3), dtype=np.float64)
-        barycentric[:, 2] = util.diagonal_dot(
+        barycentric[:, 2] = diagonal_dot(
             np.cross(edge_vectors[:, 0], w), n) / denominator
-        barycentric[:, 1] = util.diagonal_dot(
+        barycentric[:, 1] = diagonal_dot(
             np.cross(w, edge_vectors[:, 1]), n) / denominator
         barycentric[:, 0] = 1 - barycentric[:, 1] - barycentric[:, 2]
         return barycentric
 
     def method_cramer():
-        dot00 = util.diagonal_dot(edge_vectors[:, 0], edge_vectors[:, 0])
-        dot01 = util.diagonal_dot(edge_vectors[:, 0], edge_vectors[:, 1])
-        dot02 = util.diagonal_dot(edge_vectors[:, 0], w)
-        dot11 = util.diagonal_dot(edge_vectors[:, 1], edge_vectors[:, 1])
-        dot12 = util.diagonal_dot(edge_vectors[:, 1], w)
+        dot00 = diagonal_dot(edge_vectors[:, 0], edge_vectors[:, 0])
+        dot01 = diagonal_dot(edge_vectors[:, 0], edge_vectors[:, 1])
+        dot02 = diagonal_dot(edge_vectors[:, 0], w)
+        dot11 = diagonal_dot(edge_vectors[:, 1], edge_vectors[:, 1])
+        dot12 = diagonal_dot(edge_vectors[:, 1], w)
 
         inverse_denominator = 1.0 / (dot00 * dot11 - dot01 * dot01)
 
@@ -521,12 +547,9 @@ def closest_point(triangles, points):
     list of corresponding points.
 
     Implements the method from "Real Time Collision Detection" and
-    uses the same variable names as 'ClosestPtPointTriangle' to avoid
+    use the same variable names as "ClosestPtPointTriangle" to avoid
     being any more confusing.
 
-    Note that you could get a speedup by more carefully masking for
-    reducing the number of operations, but it is already pretty fast and
-    it becomes so confusing that I haven't bothered here.
 
     Parameters
     ----------
@@ -554,6 +577,10 @@ def closest_point(triangles, points):
     # which points still need to be handled
     remain = np.ones(len(points), dtype=np.bool)
 
+    # if we dot product this against a (n, 3)
+    # it is equivalent but faster than array.sum(axis=1)
+    ones = [1.0, 1.0, 1.0]
+
     # get the three points of each triangle
     # use the same notation as RTCD to avoid confusion
     a = triangles[:, 0, :]
@@ -564,8 +591,10 @@ def closest_point(triangles, points):
     ab = b - a
     ac = c - a
     ap = points - a
-    d1 = util.diagonal_dot(ab, ap)
-    d2 = util.diagonal_dot(ac, ap)
+    # this is a faster equivalent of:
+    # diagonal_dot(ab, ap)
+    d1 = np.dot(ab * ap, ones)
+    d2 = np.dot(ac * ap, ones)
 
     # is the point at A
     is_a = np.logical_and(d1 < tol.zero, d2 < tol.zero)
@@ -575,8 +604,9 @@ def closest_point(triangles, points):
 
     # check if P in vertex region outside B
     bp = points - b
-    d3 = util.diagonal_dot(ab, bp)
-    d4 = util.diagonal_dot(ac, bp)
+    d3 = np.dot(ab * bp, ones)
+    d4 = np.dot(ac * bp, ones)
+
     # do the logic check
     is_b = (d3 > -tol.zero) & (d4 <= d3) & remain
     if is_b.any():
@@ -585,7 +615,9 @@ def closest_point(triangles, points):
 
     # check if P in edge region of AB, if so return projection of P onto A
     vc = (d1 * d4) - (d3 * d2)
-    is_ab = (vc < tol.zero) & (d1 > -tol.zero) & (d3 < tol.zero) & remain
+    is_ab = ((vc < tol.zero) &
+             (d1 > -tol.zero) &
+             (d3 < tol.zero) & remain)
     if is_ab.any():
         v = (d1[is_ab] / (d1[is_ab] - d3[is_ab])).reshape((-1, 1))
         result[is_ab] = a[is_ab] + (v * ab[is_ab])
@@ -593,8 +625,8 @@ def closest_point(triangles, points):
 
     # check if P in vertex region outside C
     cp = points - c
-    d5 = util.diagonal_dot(ab, cp)
-    d6 = util.diagonal_dot(ac, cp)
+    d5 = np.dot(ab * cp, ones)
+    d6 = np.dot(ac * cp, ones)
     is_c = (d6 > -tol.zero) & (d5 <= d6) & remain
     if is_c.any():
         result[is_c] = c[is_c]
@@ -610,7 +642,9 @@ def closest_point(triangles, points):
 
     # check if P in edge region of BC, if so return projection of P onto BC
     va = (d3 * d6) - (d5 * d4)
-    is_bc = (va < tol.zero) & ((d4 - d3) > -tol.zero) & ((d5 - d6) > -tol.zero) & remain
+    is_bc = ((va < tol.zero) &
+             ((d4 - d3) > - tol.zero) &
+             ((d5 - d6) > -tol.zero) & remain)
     if is_bc.any():
         d43 = d4[is_bc] - d3[is_bc]
         w = (d43 / (d43 + (d5[is_bc] - d6[is_bc]))).reshape((-1, 1))
@@ -631,25 +665,27 @@ def closest_point(triangles, points):
 
 def to_kwargs(triangles):
     """
-    Convert a list of triangles to the kwargs for the Trimesh constructor.
+    Convert a list of triangles to the kwargs for the Trimesh
+    constructor.
 
     Parameters
     ---------
-    triangles: (n,3,3) float, triangles in space
+    triangles : (n, 3, 3) float
+      Triangles in space
 
     Returns
     ---------
-    kwargs: dict, with keys:
-                   'vertices' : (n,3) float
-                   'faces'    : (m,3) int
+    kwargs : dict
+      Keyword arguments for the trimesh.Trimesh constructor
+      Includes keys 'vertices' and 'faces'
 
     Examples
     ---------
-    mesh = trimesh.Trimesh(**trimesh.triangles.to_kwargs(triangles))
+    >>> mesh = trimesh.Trimesh(**trimesh.triangles.to_kwargs(triangles))
     """
     triangles = np.asanyarray(triangles, dtype=np.float64)
     if not util.is_shape(triangles, (-1, 3, 3)):
-        raise ValueError('Triangles must be (n,3,3)!')
+        raise ValueError('Triangles must be (n, 3, 3)!')
 
     vertices = triangles.reshape((-1, 3))
     faces = np.arange(len(vertices)).reshape((-1, 3))
